/*
 * Written by Sean Kelly
 * Placed into Public Domain
 */

module std.c.posix.sched;

public import std.c.posix.time;
public import std.c.posix.sys.types;

extern (C):

//
// Required
//
/*
struct sched_param
{
    int sched_priority (THR)
    int sched_ss_low_priority (SS|TSP)
    struct timespec sched_ss_repl_period (SS|TSP)
    struct timespec sched_ss_init_budget (SS|TSP)
    int sched_ss_max_repl (SS|TSP)
}

SCHED_FIFO
SCHED_RR
SCHED_SPORADIC (SS|TSP)
SCHED_OTHER

int sched_getparam(pid_t, sched_param*);
int sched_getscheduler(pid_t);
int sched_setparam(pid_t, sched_param*);
int sched_setscheduler(pid_t, int, sched_param*);
*/

version( linux )
{
    struct sched_param
    {
        int __sched_priority;
    }

    const auto SCHED_OTHER = 0;
    const auto SCHED_FIFO  = 1;
    const auto SCHED_RR    = 2;
    //SCHED_SPORADIC (SS|TSP)
}
else version( darwin )
{
    const auto SCHED_OTHER = 1;
    const auto SCHED_FIFO  = 4;
    const auto SCHED_RR    = 2;
    // SCHED_SPORADIC seems to be unavailable

    private const auto __SCHED_PARAM_SIZE__ = 4;

    struct sched_param
    {
        int                         sched_priority;
        byte[__SCHED_PARAM_SIZE__]  opaque;
    }
}

int sched_getparam(pid_t, sched_param*);
int sched_getscheduler(pid_t);
int sched_setparam(pid_t, sched_param*);
int sched_setscheduler(pid_t, int, sched_param*);

//
// Thread (THR)
//
/*
int sched_yield();
*/

version( linux )
{
    int sched_yield();
}
else version( darwin )
{
    int sched_yield();
}

//
// Scheduling (TPS)
//
/*
int sched_get_priority_max(int);
int sched_get_priority_min(int);
int sched_rr_get_interval(pid_t, timespec*);
*/

version( darwin )
{
    int sched_get_priority_min(int);
    int sched_get_priority_max(int);
    //int sched_rr_get_interval(pid_t, timespec*); // FIXME: unavailable?
}